;boot2.s  
;loads the application.s, switches to pmode, sets video mode
;includes irq drivers (keyboard, mouse)



%include "tlib/tmacros.inc"

bits 16
org 0x600  

	jmp start_boot2


;*******************************************************
;16 bit real mode functions must be up here

;put lba value to convert in ax 
;returns cyl, head, sect
getchs:
	xor dx,dx
	mov bx,18  ;floppy has 18 sectors per track
	div bx
	inc dx
	mov byte [sect],dl
	xor dx,dx
	mov bx,2  ;floppy has 2 heads
	div bx
	mov byte [cyl],al
	mov byte [head],dl
	ret
	
	

;put address of null terminated string in si
biosprint: 
	lodsb           ; load byte from si to al and incr 
	or al, al       ; test for 0 terminator
	jz .done_print  ; detect end of string
	mov ah, 0Eh     ; bios function writes char in al
	int 10h         ; call bios 
	jmp biosprint   ; continue until we reach 0
.done_print:
	ret
	
;*******************************************************

start_boot2:

	mov si,str1
	call biosprint

	
;load application
loadapp:

	mov ax,[lba]
	call getchs

	push 0
	pop es

	mov dl,[0x505] ;drive# 

	;the cylinder number is actually a 10bit number
	;taken from the 2 hi bits of cl plus ch
	;but for an image file no bigger than a floppy
	;1474560 is lba=2880 which is 80cyl,0hd,1sec
	;so the cyl never gets bigger than 7 bits
	mov ch,[cyl]       ;cylinder
	mov cl,[sect]      ;sector
	mov dh,[head]      ;head
	mov ah,2           ;bios Read Disc Sector 
	mov al,1           ;read 1 sector
	;every sector is loaded from disc
	;to 0:1400 then moved
	mov bx,0x1400      ;load sector to es:bx 
	int 13h            ;call bios 
	jnc movesector     ;success

	;error-reset disc drive
	mov ah,0
	mov dl,[0x505]
	int 13h
	jmp loadapp


	
	

movesector:
	
	;convert dword memory to segm:offs
	;memory counts up fom 0 to 0x10000 by 0x200 increments
	cmp dword [memory],0x10000
	jb .1

	;increment segment and zero offset
	add word [segm],0x1000  ;increment the segment
	mov word [offs],0       ;zero the offset
	mov dword [memory],0    ;zero 
	jmp .2

	;set offset=memory
.1:	mov ax,[memory]
	mov [offs],ax

	;move the sector
.2:	cld
	mov cx,512  ;move 512 bytes
	mov si,0x1400   ;0:1400
	push word [segm]
	pop es
	mov di,[offs]
	rep movsb   ;ds:di->es:di

	;show progress *****... 
	mov si,str2    
	call biosprint 


	;increment for next sector
	inc word [lba]           
	add dword [memory],512 ;bytes/sector


	

	;************************************************
	;lba_max
	;hard code the max lba sector number  
	;lba_max = sizeof(boot2) 
	;          + sizeof(tlib)
	;          + sizeof(application)
	;where sizeof=qtysectors
	;see "times" directive at bottom of 
	;boot2.s and tlib.s and application.s
	;************************************************

	cmp word[lba],7+40+100 
	jbe loadapp ;not done loading yet
	

;jmp $  



	;reset 
	push 0
	pop es


	;kill floppy drive motor
	;what if we boot from cd ??
	mov dx,0x3f2
	mov al,0
	out dx,al


	;return vga mode info function
	mov ax,0x4f01          
	mov cx,0x103              ;800x600x8bpp 
	mov di, vgamodeinfoblock  ;bios will write to this memory
	int 10h                   ;call bios



	;save bytesperscanline
	xor eax,eax
	mov ax,[vgamodeinfoblock + 16]
	mov [BPSL],eax



	;set SuperVGA graphics mode 
	;no more "call biosprint" allowed
	mov ax,0x4f02          ;VBE set graphics mode
	mov bx,0x4103          ;800x600x8bpp + LFB
	int 10h
	cmp ah,0 ;success
	jz .saveLFB
	mov si,str4
	call biosprint
	jmp $  ;stuck without graphic mode



	;save address of LFB 
.saveLFB:
	mov eax,[vgamodeinfoblock+40]
	mov [0x500],eax

	

	;we use the bios to find the bus:dev:fun number
	;of all UHCI controllers on the pci bus
	%include "krn/getuhci.s"
	
		


	;***************************************
	;end of RealMode code
	;start our conversion to pmode
	;no more bios any more
	;***************************************

.initPmode:
	cli   ;disable interrupts
	
	
	;load address of our gdt into gdt register
	lgdt [gdt_desc]

	;load address of our idt into idt register
	lidt [idt_desc]
	
		

	;a20 on 
	mov al,0d1h  ;AT
	out 64h,al
	call picpause
	mov al,3
	out 60h,al
	call picpause
	mov al,2     ;ps/2
	out 92h,al
	;did it work ??


	
	;set pmode bit
	mov eax,cr0 
	or eax,1
	mov cr0,eax
	

	;08h means 8 bytes down from start of gdt
	;is where you will find the code segment descriptor
	jmp 08h:do_pmode
	;no instructions in here
	;and no 16 bit code below this point
bits 32
do_pmode:
	
	
	;0x10=16 bytes from start of gdt is data segment descriptor
	;no longer can we use 0000: absolute segment addressing
	;segments need the offset into gdt for privaledges/size...
	;flat memory model, all segments point to same memory
	mov ax,10h  ;
	mov ds,ax   ;data segment
	mov es,ax   ;extra segment
	mov fs,ax   ;extra segment
	mov gs,ax   ;extra segment
	mov ss,ax   ;stack segment


	;move stack pointer
	;remember its an upside down segment
	;the stack grows numerically 
	;towards smaller addresses (toward our code)
	;we have a combined code/data segment that overlap
	;and the stack is at the end 
	mov esp,stackptr






	;compute sizeof video LFB and BACKBUF 
	;this number is used by swapbuffer and clearsc
	;to blit the entire contents of video memory
	;we use MMX registers and mov 64 bytes at a time
	;so the entire size of video memory is /64
	xor edx,edx
	mov eax,[BPSL] ;bytesperscanline
	mov ebx,600
	mul ebx
	mov ebx,64
	div ebx
	mov [BPSL64],eax  ; (bytesperscanline * 600)/64




	

	;initialize the ps/2 mouse
	;the placement of this code is critical
	;moving it after pic.s is not good
	;moving it up into the real mode code is not good either
	;both hang the startup giving a black screen
	;need to investigate this further
	%include "krn/mouseinit.s"




	;remap the pic hdwre interrupts
	%include "krn/pic.s"


	;enable interrupts
	sti


	;generate and load the stdpalette
	call [GENSTDPALETTE]  
	mov esi,0   ;use stdpalette
	call [SETPALETTE]  


	;set background color to our favorite cyan
	mov bh,0
	mov bl,42
	mov ch,42
	call [SETBKCOLOR]  



	;we are in pmode now so do not specify seg:off
	;jump to start of application	
	jmp 0x15000  




;*********************************************************



str1 db 'Loading application',10,13,0
str2 db '*',0
str3 db 'Bios failed to load from disc',10,13,0
str4 db 'Bios 0x4f02-Failed to set video mode',10,13,0
str5 db 'Initializing the ps/2 mouse',0
str6 db 'Remaping the pic interrupts',0
cyl  db 0
head db 0
sect db 0
memory dd 0x0        ;memory goes 0->0x10000 then back to 0 
segm dw 0x1000       ;starting segment
offs dw 0x0000       ;starting offset
stackptr equ 0x30000 ;application.s starts at 0x10000

;the floppy image lba are as follows:
;lba=0 is the boot sector boot1.s
;lba 1,2,3,4,5,6,7 is this file boot2.s
;lba=8 is the first sector of the application.s
lba dw 8



%include "krn/pit.s"         ;irq0
%include "krn/keyboard.s"    ;irq1
%include "krn/mouse.s"       ;irq12
%include "krn/interrupts.s"  ;all other irq's and isr's



vgamodeinfoblock times 256 db 0


;boot1 must load this entire file
times 512*7 - ($-$$) db 0


