;interrupts.s
;this file is included in boot2.s
;it contains the gdt, idt and default isr's & irq's
;the keyboard and mouse irq are in seperate files


;****************************************************
;           GDT
;global descriptor table
;our code and data share the same 4gig address space
;same start, same size, same privaledges (kernel ring=0)
;***************************************************

gdt_desc:
	dw gdt_end - gdt - 1  ;gdt limit/size
	dd gdt                ;gdt linear physical address

gdt:
	dd 0            ;null descriptor
	dd 0
	
gdt_code:           ;0x08 from gdt - code segment descriptor
	dw 0xffff
	dw 0
	db 0
	db 10011010b
	db 11001111b
	db 0
	
gdt_data:           ;0x10 from gdt - data segment descriptor
	dw 0xffff
	dw 0
	db 0
	db 10010010b
	db 11001111b
	db 0
	
gdt_end:


;********************************************
;          IDT
;interrupt descriptor table
;the first 32 (0-31) are software interrupts (processor exceptions)
;the next  16 (32-47) are hardware interrupts
;**********************************************

;real mode interrupt vector table descriptor
ridtr:              
	dw 0x03ff       ;limit
	dd 0            ;base


idt_desc:
	dw idt_end - idt - 1  ;idt limit/size
	dd idt                ;idt linear physical address


idt:   
	dw ir0          ;interrupt 00, div by 0 (fault)
	dw 0x08
	db 0
	db 0x8e         ;=1000110b, ring=0
	dw 0
	
	dw ird          ;interrupt 01, RESERVED by Intel 
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird          ;interrupt 02, NMI nonmaskable (interrupt)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 03, breakpoint (trap)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ir4         ;interrupt 04, overflow (trap)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ir5         ;interrupt 05, BOUND range exceeded (fault)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ir6         ;interrupt 06, invalid opcode (fault)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 07, no math coprocessor (fault)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ir8         ;interrupt 08, double fault (abort-error code)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 09, coproc segment overrun (fault)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 10, invalide tss (fault-error code)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;int 11, segment not present (fault-error code)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ir12         ;interrupt 12, stack seg (fault-error code)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ir13         ;interrupt 13, gen protection (fault-error code)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 14, page (fault-error code)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 15, RESERVED
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 16, FPU coprocessor (fault)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 17, alignment check (fault-error code)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 18, machine check (abort)
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 19, SIMD floating point (fault)
	dw 0x08
	db 0
	db 0x8e        
	dw 0



	;19-31 are reserved unnamed exceptions
	dw ird         ;interrupt 20
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 21
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 22
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 23
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 24
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 25
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 26
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 27
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 28
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 29
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 30
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw ird         ;interrupt 31
	dw 0x08
	db 0
	db 0x8e        
	dw 0


;interrupts 32-255 are User defined non-reserved
;we reprogrammed the pic to put 16 hdwre interrupts  here
;so they dont conflict with the above software ints
;pic1, irq0-irq7, port 0x20
;pic2, irq8-irq15, port 0xa0

                  ;start of PIC1 interrupts
	dw irq0       ;interrupt 32=0x20, irq0, system timer
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw irq1      ;interrupt 33, irq1, keyboard
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw irqd1       ;interrupt 34, irq2, int from 2nd pic
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw irqd1       ;interrupt 35, irq3, com2
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw irqd1       ;interrupt 36, irq4, com1
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw irqd1       ;interrupt 37, irq5, lpt2
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw irqd1        ;interrupt 38, irq6, floppy disc
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw irqd1        ;interrupt 39, irq7, lpt1
	dw 0x08
	db 0
	db 0x8e        
	dw 0

					;**************************
                    ;start of PIC2 interrupts
					;**************************
	dw irqd2        ;interrupt 40=0x28, irq8, real time clock
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw irqd2        ;interrupt 41, irq9, general i/o
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw irqd2        ;interrupt 42, irq10, general i/o
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw irqd2        ;interrupt 43, irq11, general i/o
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw irq12        ;interrupt 44, irq12, ps2 mouse
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw irqd2        ;interrupt 45, irq13, coprocessor
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw irqd2        ;interrupt 46, irq14, hard disc
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	dw irqd2        ;interrupt 47, irq15, general i/o
	dw 0x08
	db 0
	db 0x8e        
	dw 0

	;if you want kernel services
	;use 0x30 = 48
	
idt_end:





;*****************************************************
;Interrupt Service Routines
;these are processor faults/traps/aborts
;refer to "IA-32 Intel Software Developers Manual"
;Vol 3: System Programming Guide"
;for details
;abort= non-recoverable, does not always give a return address
;trap=return address points to instruction after faulty instruction
;fault=return address points to faulty instruction
;Interrupts 8,10,11,12,13,14,17 push an error code on the stack
;the others do not
;*****************************************************




ird: ;default software interrupt handler
	mov esi,_default
	call _exception
	jmp $


ir0:  ;division by 0
	;heres some code to generate this error
	;mov eax,1
	;mov ebx,0
	;mov edx,0
	;div ebx
	mov esi,_divby0  
	call _exception
	jmp $


ir4: ;overflow
	;see the INTO instruction
	mov esi,_overflow
	call _exception
	jmp $


ir5: ;out of bounds
	;see the BOUND instruction
	mov esi,_bounds
	call _exception
	jmp $


ir6: ;invalid opcode
	;you can generate this by inserting some random
	;bytes into your code with db
	;db 0x23,0x23,0x23
	mov esi,_invalop
	call _exception
	jmp $



ir8: ;Double Fault
	mov esi,_dblfalt
	call _exception
	jmp $


ir12: ;Stack fault
	mov esi,_stack
	call _exception
	jmp $



ir13: ;General Protection Fault
	mov esi,_genpro
	call _exception
	jmp $
	


;here is our general function to display the interrupt #
;and some stack contents
_exception:

	;Interrupt #
	mov ebx,200
	mov eax,0
	mov dh,RED
	mov dl,WHI
	call [PUTS] 

	mov eax,20
	mov esi,_address
	call [PUTS]   

	;print stack contents
	mov ebx,200    
	mov ecx,40      
	mov eax,[esp] ;top of stack
	call [PUTEAX]  

	mov ecx,60      
	mov eax,[esp+4]
	call [PUTEAX]  

	mov ecx,80      
	mov eax,[esp+8]
	call [PUTEAX]  

	mov ecx,100      
	mov eax,[esp+12]
	call [PUTEAX]  

	
	call [SWAPBUF]
	ret


_retaddress dd 0
_default  db  'ird:Default processor exception',0
_address  db  'Stack: esp, esp+4, esp+8, esp+12:',0
_divby0   db  'Interrupt 0-Division by zero',0
_overflow db  'Interrupt 4-Overflow',0
_bounds   db  'Interrupt 5-Out of Bounds',0
_invalop  db  'Interrupt 6-Invalid opcode',0
_dblfalt  db  'Interrupt 8-Double Fault',0
_stack    db  'Interrupt 12-Stack Fault',0
_genpro   db  'Interrupt 13-General Protection Fault',0



;********************************************************
;        hardware interrupt service routines
;********************************************************



;default handler for pic1 interrupts
irqd1:  
	cli	
	pushad

	or dword [0x50c],100000000b

	;end of interrupt (eoi) signal for pic1
	mov al,0x20
	out 0x20,al  
	popad
	sti
	iret



;default handler for pic2 interrupts
irqd2:  
	cli
	pushad

	or dword [0x50c],1000000000b
	
	;eoi
	;any pic2 interrupt must also acknowledge pic1
	mov al,0x20
	out 0x20,al  ;eoi for pic1
	out 0xa0,al  ;eoi for pic2

	popad
	sti
	iret






;note to programmers:
;your irq's must use "end of interrupt" and preserve registers
;otherwise you will have no end of frustration 
