;keyboard.s
;this file is included in boot2.s

;***********************************************
;keyboard interrupt handler
;this code converts keyboard
;scancodes to ascii bytes
;and stores the bytes at
;memory address 0x504
;it also calls a registered hook function
;whos address is stored at 0x548
;***********************************************


irq1:  
	cli	
	pushad

	in al,0x60          ;read byte at port
	mov [_scancode],al   ;save the scancode 



	;test for keyup 
	;the low 6 bits of the scan code do not change
	;for any particular key
	;from keyup to keydown
	;only bit7 changes
	;bit7 is 1 on keyup, 0 on keydown
	mov eax,[_scancode]
	test al,10000000b
	jz .key_down
	

;*************
;keyup
;*************

	;is Lshift up ?
	cmp byte [_scancode], 0xaa  ;scancode for Lshift up
	jne .1
	mov byte [_shiftdown],0     ;clear shiftdown flag
	jmp .keydone

	;is Rshift up ?
.1:	cmp byte [_scancode], 0xb6 ;scancode for Rshift up
	jne .2
	mov byte [_shiftdown],0    ;clear shiftdown flag
	jmp .keydone

	;is Ctrl up ?
.2:	cmp byte [_scancode], 0x9d
	jne near .keydone	
	mov byte [_ctrldown],0    ;clear ctrldown flag
	jmp near .keydone
	



;*************
;keydown
;*************

.key_down:

	;is Lshift down ?
	cmp byte [_scancode], 0x2a ;see scancode table
	jne .3                    ;not shift
	mov byte [_shiftdown],1    ;set shiftdown flag
	jmp .keydone
.3:

	;is Rshift down ?
	cmp byte [_scancode], 0x36
	jne .4                   ;not shift
	mov byte [_shiftdown],1   ;set shiftdown flag
	jmp .keydone
.4:

	;is Ctrl down ?
	cmp byte [_scancode], 0x1d
	jne .5                   ;not Ctrl
	mov byte [_ctrldown],1    ;set Ctrldown flag
.5:




	;************************************************
	;check for Ctrl+F12
	;this is manual process control :)
	;immediate redirection of instruction pointer
	;this will "kill" the app
	;************************************************
	cmp byte [_scancode], 0x58
	jne .6
	;we have F12
	cmp byte [_ctrldown], 1
	jne .6
	;we have Ctrl+F12
	mov al,0x20
	out 0x20,al  ;end of interrupt signal
	popad
	sti  
	;iret would take 12 bytes off stack in 32bit mode
	pop eax
	pop eax
	pop eax
	
	;jump to where ever the user wanted us to go
	;the address should have been saved dynamically 
	;at startup 
	jmp near [0x508]




	;***************************
	;convert scancode to ascii 
	;***************************
.6:	xor eax,eax
	mov al,[_scancode]
	
	;is shift down ?
	cmp byte [_shiftdown], 1
	je .upper
	mov ebx,[_ascii_lower+eax] ;scancode to ascii conversion
	jmp .save_keypress
.upper:
	mov ebx,[_ascii_upper+eax] ;scancode to ascii conversion



.save_keypress

	;may want to filter out some keypresses ??


	;here we save the ascii keypress
	mov [0x504],bl


	;keydown hook
    ;call [0x530]




.keydone:
	mov al,0x20
	out 0x20,al  ;end of interrupt signal
	popad
	sti  
	iret  





;bufsize   dd 0   ;size of buffer
_scancode  db 0   ;scancode is saved here
_shiftdown db 0   ;flag to indicate shift state
_ctrldown  db 0   ;flag to indicate ctrl state



;the values in CAPS are defined in tmacros.inc

;our keymap to convert scancode to lower case ascii
;ESC is scancode=1, 1 is scancode=2, 2 is scancode=3...
_ascii_lower:
db NIL, ESC, '1234567890-=', BKSPACE, 
db TAB, 'qwertyuiop[]', ENTER, CTRL
db 'asdfghjkl;', "'", '`', 
db SHIFT, '\', 'zxcvbnm,./', SHIFT,
db PRNTSCR, ALT, ' ', CAPS, F1, F2, F3, F4, F5, F6, F7, F8, F9, F10
db NUM, SCRLOCK, HOME, UP, PAGEUP, '-', LEFT, CENTER, RIGHT
db '+', END, DOWN, PAGEDN, INSERT, DELETE, '/', ENTER, F11, F12
;after F12 the keys transmit two scancode per keypress
;double scancode keys are not supported yet


;keymap to convert scancode to upper case ascii
;if shift key is first held down
_ascii_upper:
db NIL, ESC, '!@#$%^&*()_+', BKSPACE, 
db TAB, 'QWERTYUIOP{}', ENTER, CTRL
db 'ASDFGHJKL:"~',  
db SHIFT, '|', 'ZXCVBNM<>?', SHIFT,
db PRNTSCR, ALT, ' ', CAPS, F1, F2, F3, F4, F5, F6, F7, F8, F9, F10
db NUM, SCRLOCK, HOME, UP, PAGEUP, '-', LEFT, CENTER, RIGHT
db '+', END, DOWN, PAGEDN, INSERT, DELETE, '/', ENTER, F11, F12




