;mouse.s
;irq12 ps2 mouse driver

;this file is included in boot2.s

;this driver will call hooks into your app
;for button clicks and mouse movement
;you must register these hooks at the start of your app

;since the mouse pointer is drawn in tlib/swapbuf
;a minimal mousemove hook will "call paint"

;if you expect some graphic change in response to a mouse click
;then your button hook must also "call paint"

;this code may/maynot work for a mouse with a usb mouse
;and a usb2ps2 adapter, I have had mixed success.
;I suggest you buy a pure ps/2 mouse (6pin round connector)

;this isr is based on the fact that the bytes come 
;from the mouse 1 per interrupt 
;you may see some code elsewhere (on the net) 
;that attempts to read all 4 bytes
;from the keyboard controller on 1 interrupt
;this actually works on some hdwre 
;and it unfortunately works on Bochs
;but on other hdwre it will not work (mouse bounces around)
;the driver from Sanik (osdev) 
;and a doc from cas.mcmaster.ca/~se3f03
;suggests you read 1 byte per interrupt

_mousepacketID db 0  ;0,1,2,3 = 4 byte packet


irq12:

	cli  ;disable hdwre interrupts (can still get exception)
	pushad
	

	;use edx to decide if we need to call mousemove
	xor edx,edx


	;read mouse packet byte
	;one byte per interrupt
	call wait_read
	in al,0x60



	;store first byte of packet
	;bit0=left button
	;bit1=right button
	;bit2=middle button
	;bit3=always set
	;bit4=X sign
	;bit5=Y sign
	;bit6=X overflow
	;bit7=Y overflow
	cmp byte [_mousepacketID],0
	jnz .1
	inc byte [_mousepacketID]
	mov [0x54e],al
	jmp .eoi

.1:
	;store second byte of packet (delta X)
	cmp byte [_mousepacketID],1
	jnz .2
	inc byte [_mousepacketID]
	mov [0x54f],al
	jmp .eoi

.2:
	;store third byte of packet  (delta Y)
	cmp byte [_mousepacketID],2
	jnz .3
	inc byte [_mousepacketID]
	mov [0x550],al
	jmp .eoi

.3:
	;store fourth byte of packet  (delta Z)
	mov byte [_mousepacketID],0
	mov [0x551],al




	;now deal with the 4 bytes of the packet

	;LEFT BUTTON DOWN
	test byte [0x54e],1  ;if bit0=0 then Lbut is up and zf is set
	jz .3a
	call [LBUTHOOK] ;left button hook
	jmp .3c


.3a:	
	;RIGHT BUTTON DOWN
	test byte [0x54e],10b  ;if bit1=0 then Rbut is up and zf is set
	jz .3b
	call [RBUTHOOK] ;right button hook
	jmp .3c


.3b:
	;MIDDLE BUTTON DOWN
	test byte [0x54e],100b  ;if bit2=0 then Mbut is up and zf is set
	jz .3c
	call [MBUTHOOK] ;middle button hook





.3c:
	;MOUSEMOVE

	;get amount of delta X movement
	movsx ebx, byte [0x54f]  
	cmp ebx,0
	jz .5  ;no dx movemement
	add [MOUSEX], ebx
	mov edx,1  ;indicate we have movement
	
	
	;test for mouse outside left border 
	cmp dword [MOUSEX],0
	jge .4
	mov dword [MOUSEX],0  ;clamp


	;test for mouse outside right border
.4	cmp dword [MOUSEX],799
	jl .5
	mov dword [MOUSEX],799  ;clamp



.5:	
	;deltaY
	;when you push the mouse "up" you get a +deltaY
	;but in video memory +Y is down so we neg
	neg byte [0x550]      
	movsx ebx,byte [0x550] 
	cmp ebx,0
	jz .7  ;no dy movement	
	add [MOUSEY], ebx
	mov edx,1  ;indicate we have movement


	;test for mouse outside top border 
	cmp dword [MOUSEY],0
	jge .6
	mov dword [MOUSEY],0  ;clamp


	;test for mouse outside bottom border
.6	cmp dword [MOUSEY],599
	jl .7
	mov dword [MOUSEY],599  ;clamp



.7:	
	;if no dx and no dy then no mousemove
	cmp edx,0
	jz .8
	;every 4th mouse interrupt (end of packet)
	;need this to update the mouse pointer
	call [MMOVEHOOK]




.8:
	;deltaZ
	;nothing yet
	;we just ignore mouse wheel


	


.eoi:


	;eoi-end of interrupt signal for pic2
	mov al,0x20
	out 0xa0,al  ;eoi for pic2
	out 0x20,al  ;eoi for pic1


	popad
	sti  ;enable interrupts
	iret





