;mouseinit.s
;this file is included in boot2.s
;Routines to initialize the Mouse
;this code works for a ps/2 mouse with wheel
;which sends a 4 byte packet and returns DeviceID=0x03
;other mice are not supported
;also mice with usb connector on end and then 
;usb2ps2 connector may or may not work
;wait_write & wait_read are also used in 
;mouse.s for the irq12 mouse driver
;byte at 0x54c stores the DeviceID which should be 0x03
;byte at 0x54d stores a bitmask of errors (0=no errors)



jmp near init_mouse



;*****************************************************
;mouse_pause
;menuet adds a pause at the end of 
;wait_read and it is definitely reqd
;if you check 0x64 and it says its ok
;you must still wait for the proper response
;read too quick and you may get 0xfe from the mouse
;in bochs this pausing takes a long time
;but on real hdwre it seems necessary
;and on PII its un-noticable
;*****************************************************
mouse_pause:
	push ecx
	;there is nothing magic about 0xa0000
	;its reqd for my Toshiba laptop P1
	;faster Pentiums can use a much smaller value
	mov ecx,0xa0000
.1: loop .1

	pop ecx
	ret





;********************************************
;wait_write
;checks keyboard controller status register
;to see if INPUT buffer is empty
;this function also used by driver mouse.s
;********************************************
wait_write:
	push ecx
	mov ecx,0xa0000
.1:
	in al,0x64          ;read status reg
	test al,00000010b   ;bit1 is 0 if input buffer ready to send
	jz .2               ;success
	dec ecx
	jnz .1

	;time out error
	or byte [0x54d],100000b

.2:
	pop ecx
	ret


;********************************************
;wait_read
;checks keyboard controller status register
;to see if OUTPUT buffer is full
;this function also used by driver mouse.s
;********************************************
wait_read:
	push ecx
	mov ecx,0xa0000
.1:
	in al,0x64         ;read status reg
	test al,00000001b  ;bit0 is 0 if output buffer not ready to read
	jnz .2             ;success
	dec ecx
	jnz .1
	
	;time out error
	or byte [0x54d],1000000b

.2:
	pop ecx
	ret



;**************************************************************
;sendsendack
;sends 0xd4 to 0x64 (status port) of the keyboard controller
;which means the next byte is for the mouse 
;then sends the next byte which is relayed to the mouse
;then reads 0x60 (data port) and checks for 0xfa acknowledgement
;bl contains the command byte to send to the mouse
;**************************************************************
sendsendack:
	mov ecx,1000
.1:
	call wait_write
	mov al,0xd4  ;next byte written goes to mouse
	out 0x64,al
	
	call wait_write
	mov al,bl  
	out 0x60,al  ;send command to mouse
	
	call wait_read
	call mouse_pause
	in al,0x60   ;check port
	;mouse should acknowledge with 0xfa
	
	;i have found the mouse sometimes gives 0xfe "resend"
	;and your startup screen would hang black
	;hope the mouse gives us 0xfa soon
	cmp al,0xfa 
	jz .done
	loop .1
	
	;error
	or byte [0x54d],10000000b

.done:
	ret






init_mouse:

	;zero out 
	mov [0x54c],dword 0
	

	call wait_write
	call mouse_pause
	mov al,0xa8  ;enable mouse port
	out 0x64,al


	;code to initialize the Intellimouse
	;see Adam Chapweske "The PS/2 Mouse Interface"
	;this is by far the best reference I have found
	;ID 0x00 = 3 byte packet
	;ID 0x03 = 4 byte packet (Intellimouse)
	;ID 0x04 = 4 byte packet for 5 btn mouse + wheel
	;my irq12 driver in interrupts.s assumes 0x03


.mousereset:
	mov bl,0xff  
	call sendsendack
	cmp al,0xfa
	jnz .mousereset
	
	call wait_read
	call mouse_pause
	in al,0x60
	cmp al,0xaa
	jnz .mousereset
	
	call wait_read
	call mouse_pause
	in al,0x60
	cmp al,0
	jnz .mousereset






	;Enter scrolling wheel mode
	mov bl,0xf3   ;set sample rate
	call sendsendack
	
	mov bl,0xc8
	call sendsendack
	
	mov bl,0xf3
	call sendsendack
	
	mov bl,0x64
	call sendsendack
	
	mov bl,0xf3
	call sendsendack
	
	mov bl,0x50
	call sendsendack
	
	;now check the deviceID and see if we got 0x03

	mov bl,0xf2   ;GetDeviceID
	call sendsendack
	
	call wait_read
	call mouse_pause
	
	mov cx,1000
.checkID:	
	in al,0x60
	cmp al,0x03   ;=intelliwheel mosue
	jz .savemouseID
	loop .checkID
	
.savemouseID:
	mov [0x54c],al 



	

	;enable interrupts
.1:	call wait_write
	call mouse_pause
	mov al,0x20  
	out 0x64,al  
	call wait_read
	call mouse_pause
	in al,0x60
	or al,2
	mov bl,al   ;save status
	call wait_write
	call mouse_pause
	mov al,0x60
	out 0x64,al
	call wait_write
	call mouse_pause
	mov al,bl
	out 0x60,al


	;set defaults
	mov bl,0xf6   
	call sendsendack
	
	;enable data reporting
	;now the mouse will send data to the keyboard controller
	mov bl,0xf4   
	call sendsendack

	
	;initialize mouse position
	mov [MOUSEX],dword 400
	mov [MOUSEY],dword 300





