

;**************************************
; program the DAC
; this is where we load a palette
; DAC color values are 6 bit 
; see Michael Abrash for more info on this
; here the palette is an arry of bytes 
; of RGB triples (rrggbbrrggbb...)
; each byte ranges from 0-63 (0x3f=111111)
; the total byte count must be a multiple of 3 (rgb)
; esi = address of palette
; ecx = total bytes in palette (= qtycolors*3)
;**************************************


;your basic 16 DAC colors 
;scale for 6 bit 0-63 max
;see tmacros.inc where these colors are %defined
;some examples:
;63DAC=255RGB
;42DAC=170RGB
vga16:
db 0,0,0    ;BLACK
db 0,0,42   ;BLUE
db 0,42,0   ;GREEN
db 0,42,42  ;CYAN
db 42,0,0   ;RED
db 42,0,42  ;MAGENTA  (purple)
db 42,21,0  ;BROWN
db 42,42,42 ;LTGRAY
db 21,21,21 ;DKGRAY
db 21,21,63 ;LTBLUE
db 21,63,21 ;LTGREEN
db 21,63,63 ;LTCYAN
db 63,21,21 ;LTRED
db 63,21,63 ;LTMAGENTA
db 63,63,21 ;YELLOW
db 63,63,63 ;WHITE




;**************************************************
; setpalette
; load the DAC with a palette
; esi=address of palette
;     if esi=0 then use stdpalette
; ecx=qty bytes in palette array 
;**************************************************
setpalette:

	pushad

	mov al,0      ;starting index=0
	mov dx,0x3c8  ;control
	out dx,al
	mov dx,0x3c9  ;data

	cmp esi,0
	jnz .setpal
	;load stdpalette
	mov esi,_stdpalette
	mov ecx,768

.setpal
	mov al,[esi] 
	out dx,al 
	inc esi
	loop .setpal

	popad
	ret







;********************************************
; stdpalette
; this is our standard palette of 256 colors
; the first 216 colors are generated from
; a 6x6x6 array where each is 6bit max
; the next 22 colors are shades of gray
; the next entry is unused
; the next 16 colors are the vga16 colors
; the last entry 0xff is reserved for
; the background color
; the program bm8gen uses the same palette
;********************************************

;256 entries * 3 bytes per = 768

_stdpalette times 768 db 0
_r db 0
_g db 0
_b db 0



;********************************
;genstdpalette
;fill the _stdpalette array entries
;6x6x6=216 colors
;change red fastest
;takes no args and returns no value
;********************************
genstdpalette:
	mov eax,0 
	mov ebx,0  ;DAC color value
	mov edx,0  ;DAC color value
	mov ecx,0

.1: 
	mov [_stdpalette + ecx    ], eax  ;r
	mov [_stdpalette + ecx + 1], ebx  ;g
	mov [_stdpalette + ecx + 2], edx  ;b

	;inner loop
	add ecx,3
	add eax,12
	inc byte [_r]
	cmp byte [_r],6
	jb .1

	;middle loop
	mov byte [_r],0
	mov eax,0
	add ebx,12
	inc byte [_g]
	cmp byte [_g],6
	jb .1

	;outer loop
	mov byte [_r],0
	mov eax,0
	mov byte [_g],0
	mov ebx,0
	add edx,12
	inc byte [_b]
	cmp byte [_b],6
	jb .1


	;now append 22 shades of gray
	;starting with black (0,0,0) index=216  GRAY1
	;ending with white (63,63,63) index=237  GRAY22
	;GRAY1 thru GRAY22 are defined in tmacros.inc
	mov ebx,0  ;DAC color value
.2:
	mov [_stdpalette + ecx], ebx
	mov [_stdpalette + ecx + 1], ebx
	mov [_stdpalette + ecx + 2], ebx

	add ecx,3
	add ebx,3
	cmp ebx,64
	jb .2


	;skip a color (ends up being black in the DAC)
	;unused entry 
	add ecx,3

	;now append the vga16 colors
	mov eax,0
.3:
	mov ebx,[vga16+eax]
	mov [_stdpalette + ecx], ebx
	inc eax
	inc ecx
	cmp eax,48
	jb .3

	;the last entry 0xff should be blank
	;reserved for the bkcolor
	;may be changed using setbkcolor

	ret







;************************************
;graypalette
;generate an array of 64 shades of gray
;starting with black and ending with white
;************************************
graypalette times 192 db 0

gengraypalette:
	mov ecx,0
	mov ebx,0  ;DAC color value
.1:
	mov [graypalette + ecx], ebx
	mov [graypalette + ecx + 1], ebx
	mov [graypalette + ecx + 2], ebx

	add ecx,3
	inc ebx
	cmp ebx,64
	jb .1

	ret


