
;***************************************************
;putbits
;display a rectangular grid of pixels
;may be any width or height
;starting pixel is upper left
;uses movsb
;each pixel value is an index into the DAC (0-255)
;ebx = xstart - upper left col, pixel
;eax = ystart - upper left row, pixel
;ecx = width,pixels 
;ebp = height,pixels
;esi = address of bitmap bits
;***************************************************

putbits:
	pushad
	
	;compute address of first pixel
	MAC_GETPIXAD ebx,eax

	mov edx,[BPSL] ;bytesperscanline
	sub edx,ecx  
	;edx hold qty bytes to advance video buffer to next row

	cld         ;increment
	mov ebx,ecx ;save a copy of width

	;set pixels 1 at a time
.1:	rep movsb   ;esi->edi set entire row of pixels
	dec ebp      ;one less row
	je .2
	add edi,edx ;increment video buffer to next row
	mov ecx,ebx ;rep destroys ecx so restore for next row
	jmp .1

.2:	popad
	ret


;***********************************
;putbits4
;same as putbits except
;bitmap width must be a multiple of 4 
;put a 256 color top down bitmap to the screen
;for 800x600x8bpp graphics display
;each bit is an index into the 
;DAC 256 color palette
;the first bit is upper left corner
;the last bit is lower right corner
;the bits are arranged left to right
;top to bottom, just like the video memory
;ebx = xstart - upper left col, pixel
;eax = ystart - upper left row, pixel
;ecx = width,pixels (must be multiple of 4)
;ebp = height,pixels
;esi = address of bitmap bits
;***********************************

putbits4:
	pushad

	;compute address of first pixel
	MAC_GETPIXAD ebx,eax

	mov edx,[BPSL] ;bytesperscanline
	sub edx,ecx  
	;edx hold qty bytes to advance video buffer to next row

	cld         ;increment
	shr ecx,2   ;width/4
	mov ebx,ecx ;save width

	;set pixels 4 at a time
.1:	rep movsd   ;esi->edi set entire row of pixels
	dec ebp     ;one less row
	je .2
	add edi,edx ;increment video buffer to next row
	mov ecx,ebx ;rep destroys ecx so restore for next row
	jmp .1

.2:
	popad
	ret




;***************************************************
;puttransbits
;display a transparent rectangular grid of pixels
;the background bits 0xff are ignored
;may be any width or height
;starting pixel is upper left
;each pixel value is an index into the DAC (0-255)
;ebx = xstart - upper left col, pixel
;eax = ystart - upper left row, pixel
;ecx = width,pixels 
;ebp = height,pixels
;esi = address of bitmap bits
;***************************************************

puttransbits:
	pushad
	
	;edi gets address of first pixel in back buffer
	MAC_GETPIXAD ebx,eax

	mov edx,[BPSL] ;bytesperscanline
	sub edx,ecx  
	;edx hold qty bytes to advance video buffer to next row

	cld         ;increment
	mov ebx,ecx ;save a copy of width


.1:	cmp byte [esi],0xff  ;test for background pixel in source
	jz .2        ;skip background pixel
	movsb        ;set pixel esi->edi and increment addresses
	jmp .3
.2: inc esi
	inc edi
.3	loop .1      ;dec ecx and jmp if not 0

	dec ebp      ;one less row
	je .4
	add edi,edx ;increment video buffer to next row
	mov ecx,ebx ;loop destroys ecx so restore for next row
	jmp .1

.4:	popad
	ret



