

;****************************************************	
;putc
;display a single bitmap character 
;in 800x600x8bpp graphics mode
;origin of char is upper left
;uses the font in 9x15font.inc
;bits are packed 8 per byte in this file
;ebx = xloc,pixel 
;eax = yloc,pixel 
;ecx = ascii char to display
; dh = text color (0-0xff)
; dl = bk color (0-0xff)
;****************************************************	

_putcbitmap times 8*15 db 0 


putc:
	pushad

	;test for outside the ascii range 0x20-0x7e
	;display box char 0x7f for all nonprintable
	cmp ecx,0x20
	jb .showbox
	cmp ecx,0x7e
	ja .showbox
	jmp .loadaddress
.showbox:
	mov ecx,0x7f    ;box character


.loadaddress:
	lea ebp,[font01table + ecx*4 - 0x80]
	;ebp holds address of entry in font jump table
	;done with ecx

	mov ecx,8               ;8 pixels per row
	mov esi,[ebp]           ;address of source bitmap 
	mov edi,_putcbitmap     ;dest bitmap

	push eax ;need for counter
	push ebx ;need 8bit registers
	
	mov bl,[esi]      ;source bitmap first char
	mov bh,10000000b  ;row bitmask
	mov eax,15        ;row counter


	;set text and background color of _putcbitmap
.setrow:
	test bl,bh          ;source bits are packed 8/byte
	jz .2
	mov [edi],dh        ;set text color
	jmp .3
.2: mov [edi],dl        ;set bk color
.3:	shr bh,1            ;adjust bitmask for next bit of source
	inc edi             ;next byte of dest
	loop .setrow

	;setup for the next row of bits
	inc esi             ;point to next byte of source bitmap
	mov bh,10000000b    ;restore row bitmask
	mov ecx,8           ;restore loop counter
	mov bl,[esi]        ;get next byte of source bitmap 
	dec eax             ;15 rows per char bitmap
	jnz .setrow

	;display the bitmap char
	pop ebx
	pop eax
	mov ecx,8  ;width 
	mov ebp,15 ;height
	mov esi,_putcbitmap
	call putbits

	popad
	ret





