;******************************************************
;tlib.s
;toms library
;NASM assembly language routines 

;functions which access video memory 
;are customized for 800x600x8bpp graphics mode
;and write to a back buffer for double buffering

;functions which draw text (putc,puts,getc,gets...)
;are customized for the 9x15font.inc

;the coordinate system for graphic functions is:
;the pixel 0,0 is the upper left corner of the screen
;the pixel 799,599 is the lower right corner of the screen

;symbols for memory in tlib use a leading underscore
;functions like fillrect do not like w=0 or h=0
;there is minimal error checking

;tlib and tmacros are always changing and so is the memory mapping 
;so be careful with usage. Not every function call may be up to date
;I tend to update functions and macros as I need them.

;Tom Timmermann
;www.ticon.net/~tatimmer
;March 2008
;see /doc/LICENSE

;******************************************************

%include "tlib/tmacros.inc"

bits 32
;boot2 loads the entire library to 0x10000
org 0x10000


;here is our table of addresses for all tlib symbols 
;for example to call the gets function:
;call [0x10004]


dd getc          ;0x10000
dd gets          ;0x10004 
dd putc          ;0x10008
dd puts          ;0x1000c 
dd abs           ;0x10010 
dd atol          ;0x10014
dd putbits       ;0x10018
dd putbits4      ;0x1001c 
dd puttransbits  ;0x10020 
dd putbm8        ;0x10024
dd fillrect      ;0x10028
dd clearsc       ;0x1002c 
dd setpalette    ;0x10030
dd genstdpalette ;0x10034
dd rand          ;0x10038
dd dumpreg       ;0x1003c
dd min           ;0x10040
dd getbitmap     ;0x10044
dd setbkcolor    ;0x10048
dd strlen        ;0x1004c
dd puteax        ;0x10050
dd puteax10      ;0x10054
dd dumpmem       ;0x10058
dd gettime       ;0x1005c
dd sleep         ;0x10060
dd getvideosize  ;0x10064
dd swapbuf       ;0x10068
dd hline         ;0x1006c
dd vline         ;0x10070
dd grid          ;0x10074
dd ptinrect      ;0x10078
dd putsn         ;0x1007c
dd toggle        ;0x10080



%include "tlib/9x15font.inc"
%include "tlib/getc.s"
%include "tlib/gets.s"
%include "tlib/putc.s"
%include "tlib/puts.s"
%include "tlib/abs.s"
%include "tlib/atol.s"
%include "tlib/putbits.s"
%include "tlib/putbm8.s"
%include "tlib/fillrect.s"
%include "tlib/clearsc.s"
%include "tlib/palette.s"
%include "tlib/rand.s"
%include "tlib/dumpreg.s"
%include "tlib/min.s"
%include "tlib/cliprect.s"
%include "tlib/getbitmap.s"
%include "tlib/setbkcolor.s"
%include "tlib/strlen.s"
%include "tlib/puteax.s"
%include "tlib/dumpmem.s"
%include "tlib/gettime.s"
%include "tlib/sleep.s"
%include "tlib/getvideosize.s"
%include "tlib/swapbuf.s"
%include "tlib/line.s"
%include "tlib/grid.s"
%include "tlib/ptinrect.s"
%include "tlib/toggle.s"




;pad out to 40 sectors
times 512*40- ($-$$) db 0
