;******************************************************************
;tmacros.inc
;some defines and macros 
;some are wrappers for functions in tlib.s
;Tom Timmermann
;www.ticon.net/~tatimmer
;Feb 2008
;******************************************************************


;some names for memory addresses 


;see doc/memorymap 
%define LFB           0x500    ;linear frame buffer
%define PITCOUNTER    0x51c
%define LBUTHOOK      0x520
%define MBUTHOOK      0x524
%define RBUTHOOK      0x528
%define PAINT         0x52c
%define KEYHOOK       0x530
%define MMOVEHOOK     0x534
%define BPSL          0x538    ;bytesperscanline
%define BPSL64        0x53c    ;BPSL*600/64
%define MOUSEX        0x554
%define MOUSEY        0x558


;see tlib/tlib.s
%define GETC          0x10000
%define GETS          0x10004
%define PUTC          0x10008
%define PUTS          0x1000c
%define PUTBITS       0x10018
%define CLEARSC       0x1002c
%define FILLRECT      0x10028
%define SETPALETTE    0x10030
%define DUMPREG       0x1003c
%define GENSTDPALETTE 0x10034
%define SETBKCOLOR    0x10048
%define PUTEAX        0x10050
%define PUTEAX10      0x10054
%define DUMPMEM       0x10058
%define SLEEP         0x10060
%define SWAPBUF       0x10068
%define HLINE         0x1006c
%define VLINE         0x10070
%define PUTSN         0x1007c
%define TOGGLE        0x10080


%define BACKBUF   0x1100000  ;all tlib graphic functions draw here



;the vga16 colors 
;see palette.s for rgb values of these colors
%define BLA   239  ;ef  black
%define BLU   240  ;f0  blue
%define GRE   241  ;f1  green
%define CYA   242  ;f2  cyan
%define RED   243  ;f3  red
%define MAG   244  ;f4  magenta
%define BRN   245  ;f5  brown
%define LGR   246  ;f6  light gray
%define GRA   247  ;f7  dark gray
%define LBL   248  ;f8  light blue
%define LGN   249  ;f9  light green
%define LCY   250  ;fa  light cyan
%define LRE   251  ;fb  light red
%define LMA   252  ;fc  light magenta
%define YEL   253  ;fd  yellow
%define WHI   254  ;fe  white
;0xff is reserved for bkcolor in DAC



;shades of gray in the stdpalette
;going from dark to light
%define GRAY1  216   ;r=g=b=0  black
%define GRAY2  217   ;r=g=b=3
%define GRAY3  218   ;6
%define GRAY4  219   ;9
%define GRAY5  220   ;12
%define GRAY6  221   ;15
%define GRAY7  222   ;18
%define GRAY8  223   ;21
%define GRAY9  224   ;24
%define GRAY10 225   ;27
%define GRAY11 226   ;30
%define GRAY12 227   ;33
%define GRAY13 228   ;36
%define GRAY14 229   ;39
%define GRAY15 230   ;42
%define GRAY16 231   ;45
%define GRAY17 232   ;48
%define GRAY18 233   ;51
%define GRAY19 234   ;54
%define GRAY20 235   ;57
%define GRAY21 236   ;60
%define GRAY22 237   ;63 white



;non-displayable keydowns
;random values for identification 
;keyboard.s uses these in a lookup table
;to convert scancode to ascii
%define NIL     -1
%define ENTER   31 
%define F1      130
%define F2      131
%define F3      132
%define F4      133
%define F5      134
%define F6      135
%define F7      136
%define F8      137
%define F9      138
%define F10     139
%define F11     140
%define F12     141
%define ESC     142
%define SHIFT   143
%define CAPS    144
%define CTRL    145
%define ALT     146
%define NUM     147
%define BKSPACE 148 
%define TAB     149
%define HOME    151
%define UP      151
%define PAGEUP  152
%define LEFT    153
%define CENTER  154
%define RIGHT   155
%define END     156
%define DOWN    157
%define PAGEDN  158
%define INSERT  159
%define DELETE  160
%define PRNTSCR 161
%define SCRLOCK 162 
;the space key is just ' '




;************************************
;MEMCPY
;copy bytes from source to dest 
;arg1=address of source string
;arg2=address of destination 
;arg3=num of items to copy
;arg4=b or w or d (byte/word/dword)
;esi and esi are auto incremented 
;depending on arg4
;************************************

%macro MAC_MEMCPY 4  ;source,dest,numitems,b/w/d
	pushad
	mov esi,%1 ;source
	mov edi,%2 ;destination
	mov ecx,%3 ;count
	cld        ;increment
	rep movs%4
	popad
%endmacro


;************************************
;MEMSET
;set all items in memory to same value  
;************************************

%macro MAC_MEMSET 4  ;destmemoryaddress,sourceitem,numitems,b/w/d
	pushad
	mov edi,%1  ;destination
	mov eax,%2  ;source item to place in memory
	mov ecx,%3  ;count
	cld         ;increment
	rep stos%4  ;al->edi do ecx times
	popad
%endmacro



;******************************************
;ISDIGIT
;tests for ascii char in range 0x30-0x39 (0-9)
;put byte in al (lodsb) then call this macro
;success: zf=0
;failure: zf=1
;*******************************************

%macro MAC_ISDIGIT 0
	cmp al,0x30      ;test for ascii 0
	jb %%.1          ;too low
	cmp al,0x39      ;test for ascii 9
	ja %%.1          ;too hi
	cmp al,0         ;success, zf=0
	jmp %%.2
%%.1: 
	xor eax,eax      ;failure, zf=1
%%.2:
%endmacro



%macro MAC_PUTC 5  ;x,y,asciibyte,textcolor,bkcolor
	mov ebx,%1
	mov eax,%2
	mov ecx,%3
	mov dh,%4
	mov dl,%5
	call [PUTC]
%endmacro



%macro MAC_PUTS 5  ;x,y,buf,textcolor,bkcolor
	mov ebx,%1
	mov eax,%2
	mov esi,%3
	mov dh,%4
	mov dl,%5
	call [PUTS]
%endmacro



%macro MAC_STRLEN 1  ;buf
	mov eax,%1
	call strlen
%endmacro




%macro MAC_SETBKCOLOR 3  ;r,g,b
	mov bh,%1
	mov bl,%2
	mov ch,%3
	call [0x1004c]
%endmacro



%macro MAC_FILLRECT 5  ;x,y,w,h,color
	mov ebx,%1
	mov eax,%2
	mov ecx,%3
	mov ebp,%4
	mov esi,%5
	call [FILLRECT]
%endmacro



%macro MAC_SETSTDPALETTE 0
	call [0x10038]  ;setstdpalette
%endmacro



%macro MAC_PUTEAX10 4  ;x,y,textcolor,bkcolor
	mov ebx,%1
	mov ecx,%2
	mov dh,%3
	mov dl,%4
	call [PUTEAX10]
%endmacro


%macro MAC_PUTEAX 5  ;x,y,textcolor,bkcolor,0=eax,1=ax,2=al
	mov ebx,%1
	mov ecx,%2
	mov dh,%3
	mov dl,%4
	mov edi,%5
	call [PUTEAX]
%endmacro




;**********************************************
;GETPIXAD
;get pixel address
;computes address in video back buffer
;for a single pixel based on x,y coordinates
;for 800x600 screen
;returns address in edi
;to set a pixel:  mov byte [edi], colorindex
;**********************************************
%macro MAC_GETPIXAD 2   ;x,y
	push eax
	push edx
	
	mov edi,BACKBUF          ;start of back buffer
	mov edx,[BPSL]           ;bytesperscanline
	mov eax,%2               ;eax=y
	mul edx                  ;edx:eax = y*bpsl
	add eax,%1               ;edx:eax = y*bpsl + x
	add edi,eax              ;edi holds address of pixel in BACKBUF

	pop edx
	pop eax
%endmacro



;************************************************
;PTINRECT
;clears zf if ptinrect
;RectX1,RectY1 is the upper left corner of rect
;RectX2,RectY2 is the lower right corner of rect
;Ptx,Pty is typically the mouse [MOUSEX],[MOUSEY]
;**************************************************
%macro MAC_PTINRECT 6  ;RectX1, RectY1, RectX2, RectY2, Ptx, Pty
	mov eax,%1
	mov ebx,%2
	mov ecx,%3
	mov edx,%4
	mov esi,%5
	mov edi,%6
	call ptinrect
%endmacro



;*************************************
;MAC_SHOWMOUSEXY
;display the mouse x,y coordinate
;like this:   422  345
;*************************************
%macro MAC_SHOWMOUSEXY 4   ;x,y,textcolor,bkcolor
	mov eax,[MOUSEX]
	MAC_PUTEAX10 %1,%2,%3,%4
	mov eax,[MOUSEY]
	MAC_PUTEAX10 %1+50,%2,%3,%4
%endmacro




;**********************************************
;MAC_INC
;use this macro to increment a dword in memory
;up to some max value and no more
;**********************************************
%macro MAC_INC 2  ;dwordaddress, maxvalue
	push eax
	mov eax,[%1]
	cmp eax,%2
	jae %%done
	inc dword [%1]
%%done:
	pop eax
%endmacro



;**********************************************
;MAC_DEC
;use this macro to decrement a dword in memory
;down to zero and no less
;**********************************************
%macro MAC_DEC 1  ;dwordaddress
	push eax
	mov eax,[%1]
	cmp eax,0
	jbe %%done
	dec dword [%1]
%%done:
	pop eax
%endmacro


