
;uhcihdwre.s


;********************************************************
;resetuhcicontroller
;the controller runs all the time
;like a wild horse
;cycling thru the frame list
;a transaction occurs 
;when you connect the first td in a linked list of td's
;to a qh which is pointed to by an entry in the frame list
;at first we make all entries in the frame list
;point to the same qh
;later you can get fancy by mixing control, bulk, iso
;a transaction ends when the tds making up the transaction
;are marked "inactive" by the controller
;or marked "stall" or the 5sec timeout expires
;********************************************************

resetuhcicontroller:

	;first we HCRESET the controller
	;this stops the controller
	;also affects bits 8,3:0 of PORTSC
	mov dx,[baseadd]
	in ax,dx    ;read it in
	or ax,10b   ;enable bit1 for HCRESET
	out dx,ax   ;send it back


	;pause for 1 sec
	mov eax,1000
	call [SLEEP]


	;mark our head qh at 0x01005000 
	;1st dword of qh points to next qh (horizontal move)
	mov dword [0x01005000],1 ;terminate


	;2nd dword of qh is pointer to vertical td 
	;to initiate a usb transaction write your tdstart to here
	mov dword [0x01005000+4],1 ;terminate

	
	;fill each entry in the frame list 
	;with the address of our head qh
	cld
	mov ecx,1024
	mov eax,0x01005000 ;address of our head qh
	or eax,10b         ;item points to qh, valid ptr
	mov edi,0x01000000
	rep stosd


	;now assign 0x01000000 to FLBASEADD 
	mov dx,[baseadd]
	add dx,0x08  ;i/o address=base+8
	mov eax,0x01000000
	out dx,eax     


	;zero out FRNUM
	mov dx,[baseadd]
	add dx,0x06  ;i/o address=base+6
	in ax,dx
	and ax,1111100000000000b
	out dx,ax     


	;max packet size=64 bytes for FULL speed
	mov dx,[baseadd]
	in ax,dx
	or ax,10000000b  ;bit7=1 for 64 byte packet max
	out dx,ax


	;restart the controller
	mov dx,[baseadd]
	in ax,dx
	or ax,1  ;bit0=1 for run
	out dx,ax

	ret




;********************************************************
;resetport0
;run this after resetting the controller
;also if your communication with the device gets screwed up 
;the only solution I found is to do this:
;	*resetport0
;   *run SetAddress
;	*if SetAddress fails then resetport0 again
;now SetAddress should pass and you can SetConfig
;and use the device. 
;********************************************************

resetport0:

	;first reset PORT0 
	mov dx,[baseadd]
	add dx,0x10
	mov ax,0x200  ;bit9 is reset
	out dx,ax     

	;pause while the reset signal is active
	mov eax,50
	call [SLEEP]

	;write all zeros to the register
	mov dx,[baseadd]
	add dx,0x10
	mov ax,0
	out dx,ax     

	;pause again
	mov eax,10
	call [SLEEP]
	
	;enable PORT0
	mov dx,[baseadd]
	add dx,0x10
	mov ax,1110b  ;enablechange + enable + connectchange
	out dx,ax    

	;pause 1 more time
	mov eax,50
	call [SLEEP]

	ret



;***************************************************

readusbspace:

	;read the usb i/o space registers
	
	;read the USBCMD command register
	mov dx,[baseadd]
	in ax,dx
	mov [usbcmd],ax

	;read the USBSTS status register
	mov dx,[baseadd]
	add dx,0x02
	in ax,dx
	mov [usbsts],ax

	;read PORTSC0  port0
	mov dx,[baseadd]
	add dx,0x10
	in ax,dx
	mov [portsc0],ax

	;read PORTSC1  port1
	mov dx,[baseadd]
	add dx,0x12
	in ax,dx
	mov [portsc1],ax
	
	;read USBINTR interrupt enable register
	mov dx,[baseadd]
	add dx,0x04
	in ax,dx
	mov [usbintr],ax

	;read FRNUM frame number register
	mov dx,[baseadd]
	add dx,0x06
	in ax,dx
	mov [frnum],ax

	;read FLBASEADD frame list base address register
	mov dx,[baseadd]
	add dx,0x08
	in eax,dx
	mov [flbaseadd],eax

	ret


;********************************************************


;read pci configuration space registers

readpcispace:

	;bus#
	mov ebx,[controllernum]
	mov al,[0x560+ebx*2+1]
	mov [bus],al

	;device#
	;the dev and fun are packed into a single byte
	mov ebx,[controllernum]
	mov al,[0x560+ebx*2]
	mov bl,al ;copy
	shr bl,3  ;shift out the fun to get the device
	mov [device],bl


	;function#
	and al,111b
	mov [function],al


	;build the bus:dev:fun 32bit value for pci access
	;for now zero out the register num 
	mov ah,0x80
	mov al,[bus] 
	shl eax,16
	mov bl,[device]
	shl bl,3
	or bl,[function]
	mov ah,bl
	mov al,0  ;zero out the register
	mov [busdevfun],eax ;save for later 


	;get controller VID and DID
	mov dx,0xcf8  ;write to index port
	;send to command port of controller
	out dx,eax
	;read data port to get the VID and DID
	mov dx,0xcfc  
	in eax,dx


	;save VID and DID 
	;on my CBS home build I get:
	;VID=8086 (Intel)
	;DID=7112 (82371AB/EB/MB PIIX4 USB)
	;see Linux pci.ids file for listing of all pci devices
	mov [vid],ax
	shr eax,16
	mov [did],ax 


	;get PCICMD pci command register 
	;i get 0x0005
	mov eax,[busdevfun]
	mov al,0x04 ;address offset for PCICMD 
	mov dx,0xcf8  
	out dx,eax
	mov dx,0xcfc  
	in ax,dx    ;read PCICMD
	mov [pcicmd],ax


	;get CLASSC class code register 
	mov eax,[busdevfun]
	mov al,0x09 ;adderss offset for CLASSC
	mov dx,0xcf8  
	out dx,eax
	mov dx,0xcfc  
	in eax,dx    ;read CLASSC
	mov [classc],eax


	;get LATENCY Timer register
	mov eax,[busdevfun]
	mov al,0x0d  ;offset  
	mov dx,0xcf8  
	out dx,eax
	mov dx,0xcfc  
	in al,dx    ;read LATENCY
	mov [latency],al


	;get HEDT header type 
	;i get 00
	mov eax,[busdevfun]
	mov al,0x0e  
	mov dx,0xcf8  
	out dx,eax
	mov dx,0xcfc  
	in al,dx    ;read HEDT
	mov [hedt],al


	;get LEGSUP legacy support
	mov eax,[busdevfun]
	mov al,0xc0  
	mov dx,0xcf8  
	out dx,eax
	mov dx,0xcfc  
	in ax,dx    ;read LEGSUP
	mov [legsup],ax


	;get USBBA usb i/o space base address
	;this is the important one needed for usb transactions
	mov eax,[busdevfun]
	mov al,0x20  ;20=address offset for USBBA
	mov dx,0xcf8  
	out dx,eax
	mov dx,0xcfc  
	in eax,dx    ;read USBBA dword
	mov [usbba],eax
	;CBS returns 0x00001021
	;gateway returns 0x0000ef81
	;DELL returns 0x0000ff81


	;save the base address for usb transactions
	;this is bits0-15 of USBBA with bit0 cleared
	;i guess the bios gives us this value but we could change it
	and eax,0xfffffffe
	mov [baseadd],ax


	;get USB_RES
	;this register is on the newer 82801 controller
	mov eax,[busdevfun]
	mov al,0xc4 ;register for USB_RES 
	mov dx,0xcf8  
	out dx,eax
	mov dx,0xcfc  
	in al,dx    
	mov [usbres],al

	
	;get PIRQD interrupt routing register
	;this is function=0
	;we want PIRQD routed to irq3
	mov ah,0x80
	mov al,[bus] 
	shl eax,16
	mov bl,[device]
	shl bl,3
	or bl,0  ;fun=0
	mov ah,bl
	mov al,0x63  ;PIRQD
	;eax holds bus:dev:fun:reg
	mov dx,0xcf8  
	out dx,eax
	mov dx,0xcfc  
	in al,dx    
	mov [pirqd],al

	ret
